<isol>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <directfb.h>

static IDirectFB        *dfb;
static IDirectFBSurface *primary;
static IDirectFBSurface *foot;

static IDirectFBInputDevice *keyboard;
static IDirectFBInputBuffer *buffer;

static int screen_width;
static int screen_height;


int main (int argc, char **argv)
{
  int quit = 0;

  DFBSurfaceDescription   dsc;
  IDirectFBImageProvider *provider;

  /* Super-Interface besorgen 
     und Cooperative Level setzen. */
  DirectFBCreate (&argc, &argv, &dfb);
  dfb->SetCooperativeLevel (dfb, DFSCL_FULLSCREEN);

  /* Surface-Description ausfllen, 
     um das Primary Surface zu erhalten. */
  dsc.flags = DSDESC_CAPS;
  dsc.caps  = DSCAPS_PRIMARY;
  dfb->CreateSurface( dfb, &dsc, &primary );

  /* Gre des Surfaces bestimmen
     und Inhalt lschen. */
  primary->GetSize (primary, &screen_width, &screen_height);
  primary->FillRectangle (primary, 0, 0, screen_width, screen_height);

  /* Tastatur samt Input-Buffer besorgen. */
  dfb->GetInputDevice (dfb, DIDID_KEYBOARD, &keyboard);
  keyboard->CreateInputBuffer (keyboard, &buffer);

  /* Image-Provider fr das Bild erstellen
     und die passende Surface-Description dafr holen. */
  dfb->CreateImageProvider (dfb, DATADIR"/foot.png", &provider);
  provider->GetSurfaceDescription (provider, &dsc);

  /* Surface erstellen, um das Bild zu laden. */
  dfb->CreateSurface( dfb, &dsc, &foot );

  /* Erst hier wird das eigentliche Bild geladen. */
  provider->RenderTo (provider, foot);

  /* Den Provider brauchen wir jetzt nicht mehr. */
  provider->Release (provider);


  /* Da das Bild einen Alphakanal hat, 
     Flag setzen um ihn zu benutzen. */
  primary->SetBlittingFlags (primary, DSBLIT_BLEND_ALPHACHANNEL);

  /* Der Haupt-Thread (oder Mainloop) 
     gehrt komplett der Anwendung. */
  while (!quit) {
    DFBInputEvent event;

    /* Auf nchsten Event warten, 
       ohne die CPU zu belasten. */
    buffer->WaitForEvent (buffer);

    /* Events der Reihe nach verarbeiten. */
    while (buffer->GetEvent (buffer, &event) == DFB_OK) {
      /* Taste losgelassen?
	 Bildschirm lschen!. */
      if (event.type == DIET_KEYRELEASE)
	DFBCHECK (primary->FillRectangle(primary, 
					 0, 0, screen_width, screen_height));

      /* Taste (immer noch) gedrckt? */
      if (event.type == DIET_KEYPRESS) {
	if (event.keycode == DIKC_ESCAPE)
	  /* ... Escape ? Beenden! ... */
	  quit = 1;
	else
	  /* ... Sprite an zufllige Stelle kopieren. */
	  DFBCHECK (primary->Blit (primary, foot, NULL,
				   rand() % (screen_width - dsc.width),
				   rand() % (screen_height - dsc.height)));
      }
    }
  }


  /* Stackartig die Interfaces wieder freigeben. */
  buffer->Release (buffer);
  keyboard->Release (keyboard);
  foot->Release (foot);
  primary->Release (primary);
  dfb->Release (dfb);
  
  return 23;
}

