Attribute VB_Name = "WS_SriptLib"


' Allgemeine Scripting-Funktionsbibliothek
' (C) Holger Schwichtenberg 2000
' Kontakt: hs@windows-scripting.de
' Version 1.1
' /////////////////////////////////////////////

' auer den Explicit mit "nur VBS" markierten Routinen laufen alle auch in VB6/7/A

Option Explicit

' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
' %%%%%%%%%%%%%%%%%%%%%%%%%%% Allgemeine Hilfsroutinen
' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

' --- Globale Variablen fr Protokollierung
Dim SayAll ' Liste aller Ausgaben
Dim SayCount ' Zhler fr Ausgaben
Dim SayLog ' Dateipfad fr optionale Protokollierung in Datei
Dim DebugMode ' Debugmode True | False

' /////////////////////////////////////////////////////////////////
' //////////////////// Ausgaben
' /////////////////////////////////////////////////////////////////

' ### Anhngen eines Strings an Datei
Sub WriteTo(FilePath, text)
Dim oTX ' As Scripting.TextStream
Dim fso ' As Scripting.FileSystemObject
On Error Resume Next
Set fso = CreateObject("Scripting.FileSystemObject")
Set oTX = fso.OpenTextFile(FilePath, 8, True)  ' 8 = ForAppending
oTX.WriteLine text
oTX.Close
On Error GoTo 0
End Sub

' ### Universelle Ausgabe einer Collection oder eines Containers
Sub saycol(objcol)
Dim s
On Error Resume Next
say "Collection: " & objcol.Name
say "Anzahl Objekte: " & objcol.Count
On Error GoTo 0
s = getcol(objcol)
say Replace(s, ";", Chr(13)) 'Zeilenumbruch statt Semikolon
End Sub

' ### Informationen zum Datentyp einer Variablen
Sub ti(x)
say TypeName(x) & "(" & VarType(x) & ")"
End Sub

' ### Ausgabe eines Fehlers
Sub sayerror(s)
say "FEHLER: " & Err.Number & " (" & Err.Description & "): " & s
End Sub

' ### Bedingte Ausgabe (nur im Debugmodus)
Public Sub saydebug(s)
If DebugMode Then say s
End Sub

' ### Ausfhrliche Ausgabe
Public Sub sayex(s)
On Error Resume Next
Dim text
SayCount = SayCount + 1
text = SayCount & ". (" & Now & "): " & s
SayAll = SayAll & text & vbCr & vbLf
say text
End Sub

' /////////////////////////////////////////////////////////////////
' //////////////////// Fehlertest
' /////////////////////////////////////////////////////////////////

' ### berprfung, ob Fehler passiert ist; ggf. Protokollierung!
Function check(strStep)
If Err.Number <> 0 Then
 sayerror strStep
 Err.Clear
 check = True
Else
 If DebugMode Then say "STEP OK: " & strStep
 check = False
End If
End Function

' /////////////////////////////////////////////////////////////////
' /////////////////// Einbindung
' /////////////////////////////////////////////////////////////////

' ===Universelle Einbindung externer Skriptdateien (NUR VBS)
Function Include(skriptname)
Dim fso ' As Scripting.FileSystemObject
Dim oTX ' As Scripting.TextStream
On Error Resume Next
Set fso = CreateObject("Scripting.FileSystemObject")
If fso.FileExists(skriptname) Then
 Set oTX = fso.OpenTextFile(skriptname)
 ExecuteGlobal oTX.ReadAll
 If Err.Number <> 0 Then  ' Fehler
   Include = Err.Number & ":" & Err.Description
 Else ' kein Fehler
   Include = ""
 End If
 oTX.Close
Else  ' Datei nicht gefunden
 Include = "Datei existierte nicht!"
End If
End Function

' /////////////////////////////////////////////////////////////////
' //////////////////// COM
' /////////////////////////////////////////////////////////////////

' ### Existiert eine COM-Instanz
Function ExistsObject(moniker)
Dim obj
On Error Resume Next
Set obj = GetObject(moniker)
If obj Is Nothing Then
    ExistsObject = False
Else
    ExistsObject = True
End If
Err.Clear
On Error GoTo 0
End Function

' ### Testet, ob COM-Objekt mit CreateObject instanziiert werden kann
Function CheckCreate(progid)
Dim obj
On Error Resume Next
Set obj = CreateObject(progid)
If obj Is Nothing Then
    CheckCreate = False
Else
    CheckCreate = True
End If
Err.Clear
On Error GoTo 0
End Function

' ### Fehlertoleranter Zugriff auf ein Element einer Collection
Function getItem(Item, Key)
getItem = ""
On Error Resume Next
getItem = Item(Key)
End Function

' ### CSV-Liste der Elemente einer Collection/eines Containers
Function getcol(objcol)
Dim o ' As Object
For Each o In objcol
    CSVadd getcol, o.Name
Next
End Function

' ### Erweiterte Version von getCol (nur VBS)
Function getcolex(objcol, attribut)
Dim o ' As Object
For Each o In objcol
     CSVadd getcolex, eval("o." & attribut)
Next
End Function

' /////////////////////////////////////////////////////////////////
' //////////////////// KONVERTIERUNG
' /////////////////////////////////////////////////////////////////

' ### Fgt zwei String durch Semikolon getrennt zusammen
Function CSVadd(s1, s2)
If s1 <> "" Then s1 = s1 & ";"
s1 = s1 & s2
CSVadd = s1
End Function

' ### Hilfsroutine: Macht aus einem Array eine CSV-String
Function flat(var) ' as String
On Error Resume Next
Dim i ' As Integer
If IsArray(var) Then ' Array flachklopfen
    flat = ""
    For i = LBound(var) To UBound(var)
        CSVadd flat, var(i)
    Next
Else                ' War kein Array
    flat = var
End If
End Function

' ### Entnimmt aus einem zweidimensionalen Array einen passenden Wert
' ### Ermittelt Wert von Spalte1 anhand eines Wertes aus Spalte2
Function get_from_array(wert, feld)
Dim i ' As Integer
get_from_array = "n/a"
' --- ber alle Zeilen im Feld
For i = LBound(feld, 1) To UBound(feld, 1)
    If feld(i, 1) = wert Then ' gefunden!
        get_from_array = feld(i, 0)
        Exit For
    End If
Next
End Function

' ### Entnimmt aus einem zweidimensionalen Array passende Werte
' ### Ermittelt Werte von Spalte1 anhand eines Wertes aus Spalte2
Function get_from_array_mult(wert, feld)
Dim i ' As Integer
get_from_array_mult = ""
' --- ber alle Zeilen im Feld
For i = LBound(feld, 1) To UBound(feld, 1)
    If feld(i, 1) And wert Then
         If get_from_array_mult <> "" Then get_from_array_mult = get_from_array_mult & ";"
        get_from_array_mult = get_from_array_mult & feld(i, 0)
    End If
Next
End Function

