#!/usr/bin/perl

use strict;
use Spreadsheet::WriteExcel;


# -----  Exceldatei anlegen  -----
#
my $excel = new Spreadsheet::WriteExcel( "test.xls" );


# -----  Formate  -----
#
my $center = $excel -> addformat();
my $head   = $excel -> addformat();

$center -> set_align( "center" );
$head   -> set_align( "center" );
$head   -> set_bold();


# -----  Unterroutinen aufrufen  -----
# * Jede Unterroutine erzeugt eine eigene Excel-Arbeitsmappe.
#
fonts();    # Mappe 'Fonts'
farben();   # Mappe 'Farben'
muster();   # Mappe 'Muster'
align();    # Mappe 'Ausrichtung'


# -----  Output beenden  -----
#
$excel -> close();


# -----  fonts()  -----
# * Darstellung einiger ausgewhlter Fonts.
#
sub fonts
{   my $sheet = $excel -> addworksheet( "Fonts" );

    $sheet -> set_column( 0, 0, 30 );
    $sheet -> set_column( 1, 1, 10 );

    $sheet -> write( 0, 0, "Font",    $head );
    $sheet -> write( 0, 1, "Gre", $head );

    my @fonts = ( [ 10, "Arial" ],
                  [ 12, "Arial" ],
                  [ 14, "Arial" ],
                  [ 12, "Arial Black" ],
                  [ 12, "Arial Narrow" ],
                  [ 12, "Century Schoolbook" ],
                  [ 12, "Courier" ],
                  [ 12, "Courier New" ],
                  [ 12, "Garamond" ],
                  [ 12, "Impact" ],
                  [ 12, "Lucida Handwriting" ],
                  [ 12, "Times New Roman" ],
                  [ 12, "Symbol" ],
                  [ 12, "Wingdings" ],
                  [ 12, "Ein nichtexistenter Font" ],
                );

    for (my $i=0; $i<=$#fonts; $i++)
    {   my $format = $excel -> addformat();

        $format -> set_size( @fonts->[$i]->[0] );
        $format -> set_font( @fonts->[$i]->[1] );

        $sheet -> write( $i+1, 0, @fonts->[$i]->[1], $format );
        $sheet -> write( $i+1, 1, @fonts->[$i]->[0], $format );
    }
}


# -----  farben()  -----
# * Erzeugung der Farbtafeln auf Namens- und Indexbasis.
#
sub farben
{   my $sheet = $excel -> addworksheet( "Farben" );

    $sheet -> write( 0, 0, "Name",  $head );
    $sheet -> write( 0, 1, "Farbe", $head );
    $sheet -> write( 0, 3, "Index", $head );
    $sheet -> write( 0, 4, "Farbe", $head );

    my @colors = qw( aqua black bl fuchsia gray green lime
                     navy orange purple red silver white yellow
                   );

    foreach (0..$#colors)
    {   my $format = $excel -> addformat();

        $format -> set_color( $colors[$_] );
        $format -> set_align( "center" );
        $format -> set_bold();

        $sheet -> write( $_+1, 0, $colors[$_], $center );
        $sheet -> write( $_+1, 1, $colors[$_], $format );
    }

    foreach (8..63)
    {   my $format = $excel -> addformat();

        $format -> set_color( $_ );
        $format -> set_align( "center" );
        $format -> set_bold();

        $sheet -> write( $_-7, 3, $_,      $center );
        $sheet -> write( $_-7, 4, "Farbe", $format );
    }
}


# -----  muster()  -----
# * Darstellung der mglichen Muster mit grnem Vorder- und silbrigem
#   Hintergrund.
#
sub muster
{   my $sheet = $excel -> addworksheet( "Muster" );

    $sheet -> set_column( 3, 3, 20 );

    $sheet -> write( 0, 0, "Index",  $head );
    $sheet -> write( 0, 1, "Muster", $head );
    $sheet -> write( 0, 3, "Vordergrund 'green'",  $head );
    $sheet -> write( 1, 3, "Hintergrund 'silver'", $head );

    foreach (0..31)
    {   my $format = $excel -> addformat();

        $format -> set_pattern( $_ );
        $format -> set_bg_color( "silver" );
        $format -> set_fg_color( "green" );
        $format -> set_align( "center" );

        $sheet -> write( 2*($_+1), 0, $_, $center );
        $sheet -> write( 2*($_+1), 1, "Muster $_", $format );
    }
}


# -----  align()  -----
# * Vertikale und horizontale Ausrichtung von Zelleninhalten.
# * Darstellung gedrehter Schrift.
#
sub align
{   my $sheet = $excel -> addworksheet( "Ausrichtung" );

    $sheet -> set_row( 0, 40 );

    $sheet -> write( 0, 0, "Vertikal",   $head );
    $sheet -> write( 2, 0, "Horizontal", $head );
    $sheet -> write( 4, 0, "Rotation",   $head );

    my @verti = qw( top bottom vcenter vjustify );
    my @hori  = qw( left right center fill justify );

    for (my $i=0; $i<=$#verti; $i++)
    {   my $format = $excel -> addformat();
        $format -> set_align( $verti[$i] );
        $sheet -> write( 0, $i+1, $verti[$i], $format );
    }

    for (my $i=0; $i<=$#hori; $i++)
    {   my $format = $excel -> addformat();
        $format -> set_align( $hori[$i] );
        $sheet -> write( 2, $i+1, $hori[$i], $format );
    }

    foreach (1..3)
    {   my $format = $excel -> addformat();
        $format -> set_rotation( $_ );
        $sheet -> write( 4, $_, "Rotation $_", $format );
    }

    my $format1 = $excel -> addformat();
    $format1 -> set_text_wrap();
    $sheet -> write( 0, $#verti+2, "Text\nmit\nUmbruch", $format1 );

    my $format2 = $excel -> addformat();
    $format2 -> set_align( "merge" );
    $sheet -> write( 2, $#hori+2, "Verbundene Zellen", $format2 );
    $sheet -> write_blank( 2, $#hori+3, $format2 );
}
