#!/usr/local/bin/perl

use CGI::Pretty qw/:standard :html3 :cgi-lib -no_xhtml -oldstyle_urls/;
use strict;
use Spreadsheet::WriteExcel;
use Date::Calc qw(Date_to_Days Delta_Days);

use vars qw ($sheet $zeile $text_f);

my $sheetbase = "/tmp/rk_";
my $ueber_pausch   = "39"; # Pauschale fr bernachtungskosten
my $kuerzung_frueh = "9" ; # Krzung fr Frhstck im Hotelpreis
my $km_pausch      = "0.52"; # Kilometerpauschale
my %pausch = ( "8" => "10",
	       "14" => "20",
	       "24" => "46");
my @pausch_sprung = sort { $a <=> $b } keys %pausch;

my $q = new CGI;

if ( $q->param()) {
  write_sheet();
} else {
  write_form();
}

sub write_form {

  my @table;

  push @table, TR(td(["Vorname",
		      textfield(-name => "vname",
				-size => 20,
				-maxlength => 20) . 
		      "&nbsp;Nachname&nbsp;" . 
		       textfield(-name => "nname",
				 -size => 20,
				 -maxlength => 20)  . 
		      "&nbsp;PNr.:&nbsp;".
		      textfield(-name => "pnr",
				-size => 4,
				-maxlength => 4)]));
  push @table, TR(td(["Reiseziel",
		      textfield(-name => "ziel",
				-size => 20,
				-maxlength => 20) . 
		      "&nbsp;Reisezweck&nbsp;" . 
		       textfield(-name => "zweck",
				 -size => 30,
				 -maxlength => 30)]));
  push @table, TR(td(["Reisebeginn",
		      datum("beginn")."&nbsp;".zeit("beginn")]));
  push @table, TR(td(["Reiseende",
		      datum("ende")."&nbsp;".zeit("ende")]));
  push @table, TR(td({colspan => 2},b("Fahrtkosten")));
  push @table, TR(td(["Flug",
		       textfield(-name => "flug",
				 -size => 20,
				 -maxlength => 100)]));
  push @table, TR(td(["Bahn",
		       textfield(-name => "bahn",
				 -size => 20,
				 -maxlength => 100)]));
  push @table, TR(td(["PNV/Taxi",
		       textfield(-name => "taxi",
				 -size => 20,
				 -maxlength => 100)]));
  push @table, TR(td(["PKW (km!)",
		       textfield(-name => "pkw",
				 -size => 20,
				 -maxlength => 100)]));

  push @table, TR(td({colspan => 2},b("bernachtungskosten")));
  autoEscape(undef);
  push @table, TR(td([radio_group(-name => "hotel",
				  -rows => 2,
				  -default => 'Einzelnachweis',
				  -values => [
					      'Einzelnachweis',
					      'Pauschal' ,
					     ]
				 ),
		      textfield(-name => "hotel_einzeln",
				 -size => 20,
				 -maxlength => 100) . "&nbsp;" .
		      checkbox(-name => "hotel_fruehstuck",
			      -label => "&nbsp;Inkl. Frhstck",
			      -checked => 0,
			     -value => "1")]));
  push @table, TR(td({colspan => 2},b("Bewirtungskosten")));
  push @table, TR(td(["Einzelnachweise",
		       textfield(-name => "bewirtung",
				 -size => 20,
				 -maxlength => 100) . "&nbsp;" .
		      checkbox(-name => "bewirtung_ag",
			       -label => "&nbsp;Kosten trgt Arbeitgeber",
			       -checked => 0,
			       -value => "1")
		     ]));
  autoEscape(1);

  push @table, TR(td({colspan => 2},b("Nebenkosten")));
  push @table, TR(td(["Einzelnachweise",
		       textfield(-name => "nebenkosten",
				 -size => 20,
				 -maxlength => 100)]));

  push @table, TR(td(["Vorschuss",
		       textfield(-name => "vorschuss",
				 -size => 20,
				 -maxlength => 100)]));
		      
  print header, start_html(-title => "Reisekosten Inland",
			  -bgcolor => "#CCCCCC"),
  start_form(-name => "RKForm",
	     -method => "GET"),
  table(@table);
  
  print submit(-name => "Abschicken"),end_form, end_html;
	     
}

sub datum {
  my ($name) = @_;
  my ($jahr) = (localtime())[5];
  $jahr += 1900;
  return popup_menu(-name => $name . "tag",
		    -values => [ '1' .. '31' ]) .
		      b(".").popup_menu(-name => $name . "monat",
				   -values => [ '1' .. '12']) .
                      b(".").popup_menu(-name => $name . "jahr",
				     -values => [ $jahr-2 .. $jahr ],
				     -default => $jahr);
}

sub zeit {
  my ($name) = @_;
  return popup_menu(-name => $name . "stunde",
		    -values => [ '0' .. '23']) . b(":") .
		      popup_menu(-name => $name . "minute",
				 -values => ['00' , '15', '30', '45']);
}

sub write_sheet {

  #
  # Fehlerprfung - sollte in Javascript passieren
  #
  return unless (param("nname") && param("vname") &&
		param("ziel") && param("zweck"));

  my $filename = $sheetbase . param("nname") .".xls";
  my $excel = new Spreadsheet::WriteExcel($filename);

  $text_f     = $excel->addformat();
  my $head1_f = $excel->addformat();
  my $head2_f = $excel->addformat();
  my $date_f  = $excel->addformat();
  my $time_f  = $excel->addformat();
  my $money_f  = $excel->addformat();
  my $times_f   = $excel->addformat();
  my (@fahrt, @verpflegung, @uebernachtung);

  $head1_f->set_bold();
  $head1_f->set_font("Arial");
  $head1_f->set_size("14");
  $head1_f->set_underline();
  
  $head2_f->set_bold();
  $head2_f->set_font("Arial");
  $head2_f->set_size("10");
  
  $text_f->copy($head2_f);
  $text_f->set_bold(0);

  $date_f->copy($text_f);
  $date_f->set_num_format('dd.mm.yyyy');

  $time_f->copy($text_f);
  $time_f->set_num_format('hh:mm');

  $money_f->copy($text_f);
  $money_f->set_num_format('#,##0.00 DM');

  $times_f->copy($text_f);
  $times_f->set_format("0 x");

  my @tabelle;
  $sheet = $excel->addworksheet();
  $zeile = 0;
  $sheet->write($zeile,0,"Reisekostenabrechnung",$head1_f);
  $zeile = 2;
  beschriftung("Vorname, Nachname:" . 
	       param("vname"). " ".param("nname"));
  $zeile++;
  beschriftung("Abteilung: iX-Redaktion");
  $zeile += 2;
  beschriftung("Reisebeginn am");
  $sheet->write($zeile,1,excel_date("beginn"),$date_f);
  $sheet->write($zeile,2,excel_time("beginn"),$time_f);
  $zeile++;
  beschriftung("Reiseende am");
  $sheet->write($zeile,1,excel_date("ende"),$date_f);
  $sheet->write($zeile,2,excel_time("ende"),$time_f);
  $zeile++;
  beschriftung("Zweck der Reise: ".param("zweck"));
  $zeile++;
  beschriftung("Reiseziel: ".param("ziel"));
  $zeile+=2;
  $sheet->write($zeile,0,"Fahrtkosten",$head2_f);
  $zeile++;
  beschriftung("Flugkosten lt. Anlage:");
  $sheet->write($zeile,3,param("flug"),$money_f) if param("flug");
  $fahrt[0] = $zeile;
  $zeile++;
  beschriftung("Bahnkosten lt. Anlage:");
  $sheet->write($zeile,3,param("bahn"),$money_f) if param("bahn");
  $zeile++;
  beschriftung("Sonstige Fahrtkosten lt. Anlage:");
  my $taxi = param("taxi");
  $sheet->write($zeile,3,$taxi,$money_f) if $taxi;
  $zeile++;
  $sheet->write($zeile,0,"Kilometergeld:",$text_f);
  if (param("pkw")) {
    $sheet->write($zeile,1,param("pkw"),$times_f);
    $sheet->write($zeile,2,$km_pausch,$money_f);
    $sheet->write_formula($zeile,3,
			  "=".rowcol_to_cell($zeile,1)."*$km_pausch",
			  $money_f);
  }
  $fahrt[1] = $zeile;
  $zeile += 2;
  $sheet->write($zeile,0,"bernachtungskosten",$head2_f);
  $sheet->write($zeile,1,"Anzahl Tage",$text_f),
  $zeile++;
  $uebernachtung[0] = $zeile;
  beschriftung("Einzelnachweis lt. Anlage (m. Frhstck)");
  $sheet->write($zeile,3,param("hotel_einzeln"),$money_f) 
    if (param("hotel_einzeln") && param("hotel_frhstck") 
	&& param("hotel") ne "Pauschal");
  $zeile++;
  beschriftung("Einzelnachweis lt. Anlage (o. Frhstck)");
  $sheet->write($zeile,3,param("hotel_einzeln"),$money_f) 
    if (param("hotel_einzeln") && !param("hotel_frhstck") 
	&& param("hotel") ne "Pauschal");
  $zeile++;
  beschriftung("Pauschal");
  if (param("hotel") eq "Pauschal") {
    $sheet->write($zeile,1,tage_gesamt(),$times_f);
    $sheet->write($zeile,2,$ueber_pausch, $money_f);
    $sheet->write($zeile,3,
		  "=".rowcol_to_cell($zeile,1)."*".rowcol_to_cell($zeile,2),
		  $money_f);
  }
  $uebernachtung[1] = $zeile;
  $zeile++;
  $sheet->write($zeile,0,"Verpflegungspauschale",$head2_f);
  $sheet->write($zeile,1,"Anzahl Tage",$text_f);
  $sheet->write($zeile,2,"Pauschale",$text_f);
  $zeile++;
  $verpflegung[0] = $zeile;

  my $count;
  foreach my $h (@pausch_sprung) {
    if ($h == 24) {
      $count = tage_gesamt() - 1;
      beschriftung("Tglich fr 24 Std.");
    } else {
      $count = (erster_tag() == $h ? 1:0) + 
      (letzter_tag() == $h ? 1: 0);
      beschriftung("Tglich ber $h Std.");

    }
    if ($count) {
      $sheet->write($zeile,1,$count,$times_f);
      $sheet->write($zeile,2,$pausch{$h},$money_f);
      $sheet->write($zeile,3,
		    "=".rowcol_to_cell($zeile,1)."*".rowcol_to_cell($zeile,2),
		    $money_f);    
    }
    $zeile++;
  }
  
  beschriftung("Krzung Frhstckskosten, wenn in Hotelkosten enthalten");
  if (param("hotel_fruehstueck") && ! param("hotel_pauschal")) {
    $sheet->write($zeile, 1, tage_gesamt(), $times_f);
    $sheet->write($zeile, 2, $kuerzung_frueh, $money_f);
    $sheet->write($zeile, 3, 
		  "=0.0-".rowcol_to_cell($zeile,1)."*".
		      rowcol_to_cell($zeile,2),
		  $money_f);
  }
  $verpflegung[1] = $zeile;

  $zeile += 3;
  beschriftung("Reisekosten gesamt");
  $sheet->write($zeile,3,"=SUM(".
		rowcol_to_cell($fahrt[0],3).":".
		rowcol_to_cell($verpflegung[1],3) . ")",$money_f);
  $zeile++;
  beschriftung("abzgl. Vorschuss");
  $sheet->write($zeile,3,param("vorschuss"),$money_f) if param("vorschuss");
  $zeile++;
  beschriftung("Auszahlung(+)/Rckzahlung(-)");
  $sheet->write($zeile,3,"=" .
		rowcol_to_cell($zeile-2,3). "-" .
		rowcol_to_cell($zeile-1,3),$money_f);
  $excel->close();
  print header, start_html(-title => "Reisekosten Inland",
			  -bgcolor => "#CCCCCC"),
  h3("Sie werden einen einfachen Controller sehr glcklich machen."),
  br("Ihr Reisekostenformular wurde erfolgreich erstellt.") ,
  br("Sie finden es hier: $filename"),
  end_html;
  
}

sub beschriftung {
  my $was = shift;
  $sheet->write($zeile,0,$was,$text_f);
}

sub excel_date{
  my ($prefix) = @_;
  my $string = "=DATE(" .
    join(',',((param($prefix."jahr") || 2000) - 1900,
	      param($prefix."monat") || 1,
	      param($prefix."tag") || 1)) . ")";
  return $string;
}

sub excel_time{
  my ($prefix) = @_;
  my $string = "=TIME(" .
    join(',', (param($prefix."stunde") || 0,
	       param($prefix."minute") || 0)) . ",0)";
  return $string;
}

sub tage_gesamt {
  return Delta_Days(param("beginnjahr"), param("beginnmonat"),
		    param("beginntag"),
		    param("endejahr"), param("endemonat"),
		    param("endetag"));
}

sub erster_tag {
  my $minutes;
  if (tage_gesamt) {
    $minutes = (24 - param("beginnstunde")) * 60 - param("beginnminute");
  } else {
    $minutes = param("beginnstunde") * 60 + param("beginnminute");
  }
  foreach my $p (reverse @pausch_sprung) {
    return $p if ($minutes >= $p * 60);
  } 
  return 0;
}

sub letzter_tag {
  my $minutes = 0;
  if (tage_gesamt) {
    $minutes = param("endestunde") * 60 + param("endeminute");
  } 

  foreach my $p (reverse @pausch_sprung) {
    return $p if ($minutes >= $p * 60);  
  }
  return 0;
}

#
# Das Folgende ist aus einem Beispiel der
# Spreadsheet::WriteExcel-Distribution gestohlen. ck
#
# Convert a zero based row and column reference to a A1 reference. For example
# (0, 2) to C1. $row_absolute, $col_absolute are optional. They are boolean
# values used to indicate if the row or column value is absolute, i.e. if it is
# prefixed by a $ sign: eg. (0, 2, 0, 1) converts to $C1.
#
# Returns: a cell reference string.
#
sub rowcol_to_cell {

    my $row     = $_[0];
    my $col     = $_[1];
    my $row_abs = $_[2] || 0;
    my $col_abs = $_[3] || 0;


    if ($row_abs) {
        $row_abs = '$' #'
    }
    else {
      $row_abs = ''
    }
    
    if ($col_abs) {
      $col_abs = '$' #'
    }
    else {
      $col_abs = ''
    }
    
    
    my $int  = int ($col / 26);
    my $frac = $col % 26 +1;

    my $chr1 ='';
    my $chr2 ='';


    if ($frac != 0) {
        $chr2 = chr (ord('A') + $frac -1);;
    }

    if ($int > 0) {
        $chr1 = chr (ord('A') + $int  -1);
    }

    $row++;     # Zero index to 1-index

    return $col_abs . $chr1 . $chr2 . $row_abs. $row;
}
